import { CharacteristicValue, PlatformAccessory, Service } from 'homebridge';
import { Subject } from 'rxjs';
import { HoneywellHomePlatform } from '../platform';
import * as settings from '../settings';
/**
 * Platform Accessory
 * An instance of this class is created for each accessory your platform registers
 * Each accessory may expose multiple services of different service types.
 */
export declare class RoomSensors {
    private readonly platform;
    private accessory;
    readonly locationId: settings.location['locationID'];
    device: settings.device & settings.devicesConfig;
    sensorAccessory: settings.sensorAccessory;
    readonly group: settings.T9groups;
    service: Service;
    temperatureService?: Service;
    occupancyService?: Service;
    humidityService?: Service;
    StatusLowBattery: CharacteristicValue;
    OccupancyDetected: CharacteristicValue;
    CurrentTemperature: CharacteristicValue;
    CurrentRelativeHumidity: CharacteristicValue;
    TemperatureDisplayUnits: CharacteristicValue;
    accessoryId: number;
    roomId: number;
    action: string;
    deviceLogging: string;
    deviceRefreshRate: number;
    SensorUpdateInProgress: boolean;
    doSensorUpdate: Subject<void>;
    constructor(platform: HoneywellHomePlatform, accessory: PlatformAccessory, locationId: settings.location['locationID'], device: settings.device & settings.devicesConfig, sensorAccessory: settings.sensorAccessory, group: settings.T9groups);
    /**
     * Parse the device status from the honeywell api
     */
    parseStatus(): Promise<void>;
    /**
     * Asks the Honeywell Home API for the latest device information
     */
    refreshStatus(): Promise<void>;
    /**
     * Updates the status for each of the HomeKit Characteristics
     */
    updateHomeKitCharacteristics(): Promise<void>;
    apiError(e: any): Promise<void>;
    honeywellAPIError(e: any): Promise<void>;
    /**
     * Converts the value to celsius if the temperature units are in Fahrenheit
     */
    toCelsius(value: number): number;
    config(device: settings.device & settings.devicesConfig): Promise<void>;
    refreshRate(device: settings.device & settings.devicesConfig): Promise<void>;
    logs(device: settings.device & settings.devicesConfig): Promise<void>;
    /**
     * Logging for Device
     */
    infoLog(...log: any[]): void;
    warnLog(...log: any[]): void;
    errorLog(...log: any[]): void;
    debugLog(...log: any[]): void;
    enablingDeviceLogging(): boolean;
}
//# sourceMappingURL=roomsensors.d.ts.map