import { CharacteristicValue, PlatformAccessory, Service } from 'homebridge';
import { Subject } from 'rxjs';
import { HoneywellHomePlatform } from '../platform';
import * as settings from '../settings';
/**
 * Platform Accessory
 * An instance of this class is created for each accessory your platform registers
 * Each accessory may expose multiple services of different service types.
 */
export declare class RoomSensorThermostat {
    private readonly platform;
    private accessory;
    readonly locationId: settings.location['locationID'];
    device: settings.device & settings.devicesConfig;
    sensorAccessory: settings.sensorAccessory;
    readonly group: settings.T9groups;
    service: Service;
    TargetTemperature: CharacteristicValue;
    CurrentTemperature: CharacteristicValue;
    CurrentRelativeHumidity: CharacteristicValue;
    TemperatureDisplayUnits: CharacteristicValue;
    TargetHeatingCoolingState: CharacteristicValue;
    CurrentHeatingCoolingState: CharacteristicValue;
    CoolingThresholdTemperature: CharacteristicValue;
    HeatingThresholdTemperature: CharacteristicValue;
    modes: settings.modes;
    action: string;
    roompriority: any;
    honeywellMode: Array<string>;
    deviceFan: settings.FanChangeableValues;
    deviceLogging: string;
    deviceRefreshRate: number;
    thermostatUpdateInProgress: boolean;
    doThermostatUpdate: Subject<void>;
    roomUpdateInProgress: boolean;
    doRoomUpdate: Subject<void>;
    fanUpdateInProgress: boolean;
    doFanUpdate: Subject<void>;
    constructor(platform: HoneywellHomePlatform, accessory: PlatformAccessory, locationId: settings.location['locationID'], device: settings.device & settings.devicesConfig, sensorAccessory: settings.sensorAccessory, group: settings.T9groups);
    /**
     * Parse the device status from the honeywell api
     */
    parseStatus(): Promise<void>;
    /**
     * Asks the Honeywell Home API for the latest device information
     */
    refreshStatus(): Promise<void>;
    /**
     * Asks the Honeywell Home API for the latest device information
     */
    refreshSensorStatus(): Promise<void>;
    refreshRoomPriority(): Promise<void>;
    /**
     * Pushes the requested changes for Room Priority to the Honeywell API
     */
    pushRoomChanges(): Promise<void>;
    /**
     * Pushes the requested changes to the Honeywell API
     */
    pushChanges(): Promise<void>;
    /**
     * Updates the status for each of the HomeKit Characteristics
     */
    updateHomeKitCharacteristics(): Promise<void>;
    apiError(e: any): Promise<void>;
    honeywellAPIError(e: any): Promise<void>;
    setTargetHeatingCoolingState(value: CharacteristicValue): Promise<void>;
    setHeatingThresholdTemperature(value: CharacteristicValue): Promise<void>;
    setCoolingThresholdTemperature(value: CharacteristicValue): Promise<void>;
    setTargetTemperature(value: CharacteristicValue): Promise<void>;
    setTemperatureDisplayUnits(value: CharacteristicValue): Promise<void>;
    /**
     * Converts the value to celsius if the temperature units are in Fahrenheit
     */
    toCelsius(value: number): number;
    /**
     * Converts the value to fahrenheit if the temperature units are in Fahrenheit
     */
    toFahrenheit(value: number): number;
    TargetState(): number[];
    config(device: settings.device & settings.devicesConfig): Promise<void>;
    refreshRate(device: settings.device & settings.devicesConfig): Promise<void>;
    logs(device: settings.device & settings.devicesConfig): Promise<void>;
    /**
     * Logging for Device
     */
    infoLog(...log: any[]): void;
    warnLog(...log: any[]): void;
    errorLog(...log: any[]): void;
    debugLog(...log: any[]): void;
    enablingDeviceLogging(): boolean;
}
//# sourceMappingURL=roomsensorthermostats.d.ts.map