import { AxiosInstance } from 'axios';
import { API, Characteristic, DynamicPlatformPlugin, Logger, PlatformAccessory, Service } from 'homebridge';
import * as settings from './settings';
/**
 * HomebridgePlatform
 * This class is the main constructor for your plugin, this is where you should
 * parse the user config and discover/register accessories with Homebridge.
 */
export declare class HoneywellHomePlatform implements DynamicPlatformPlugin {
    readonly log: Logger;
    readonly config: settings.HoneywellPlatformConfig;
    readonly api: API;
    readonly Service: typeof Service;
    readonly Characteristic: typeof Characteristic;
    readonly accessories: PlatformAccessory[];
    axios: AxiosInstance;
    locations?: any;
    firmware: settings.accessoryAttribute['softwareRevision'];
    sensorAccessory: settings.sensorAccessory;
    version: any;
    sensorData: never[];
    private refreshInterval;
    debugMode: boolean;
    action: string;
    platformLogging: string;
    constructor(log: Logger, config: settings.HoneywellPlatformConfig, api: API);
    logs(): void;
    /**
     * This function is invoked when homebridge restores cached accessories from disk at startup.
     * It should be used to setup event handlers for characteristics and update respective values.
     */
    configureAccessory(accessory: PlatformAccessory): void;
    /**
     * Verify the config passed to the plugin is valid
     */
    verifyConfig(): void;
    refreshAccessToken(): Promise<void>;
    /**
     * Exchange the refresh token for an access token
     */
    getAccessToken(): Promise<void>;
    /**
     * The refresh token will periodically change.
     * This method saves the updated refresh token in the config.json file
     * @param newRefreshToken
     */
    updateRefreshToken(newRefreshToken: string): Promise<void>;
    /**
     * this method discovers the Locations
     */
    discoverlocations(): Promise<any>;
    /**
     * this method discovers the rooms at each location
     */
    getCurrentSensorData(device: settings.device & settings.devicesConfig, group: settings.T9groups, locationId: settings.location['locationID']): Promise<any>;
    private normalizeSensorDate;
    /**
     * this method discovers the firmware Veriosn for T9 Thermostats
     */
    getSoftwareRevision(locationId: settings.location['locationID'], device: settings.device & settings.devicesConfig): Promise<any>;
    /**
     * This method is used to discover the your location and devices.
     * Accessories are registered by either their DeviceClass, DeviceModel, or DeviceID
     */
    private discoverDevices;
    private deviceClass;
    private discoverRoomSensors;
    private roomsensordisplaymethod;
    private createThermostat;
    private Leak;
    private createRoomSensors;
    private createRoomSensorThermostat;
    thermostatFirmwareNewAccessory(device: settings.device & settings.devicesConfig, accessory: PlatformAccessory, location: any): Promise<void>;
    thermostatFirmwareExistingAccessory(device: settings.device & settings.devicesConfig, existingAccessory: PlatformAccessory, location: any): Promise<void>;
    unregisterPlatformAccessories(existingAccessory: PlatformAccessory): void;
    locationinfo(location: settings.location): void;
    deviceinfo(device: {
        deviceID: string;
        deviceType: string;
        deviceClass: string;
        deviceModel: string;
        priorityType: string;
        settings: settings.Settings;
        inBuiltSensorState: settings.inBuiltSensorState;
        groups: settings.device['groups'] & settings.devicesConfig;
    }): void;
    apiError(e: any): void;
    /**
     * If device level logging is turned on, log to log.warn
     * Otherwise send debug logs to log.debug
     */
    infoLog(...log: any[]): void;
    warnLog(...log: any[]): void;
    errorLog(...log: any[]): void;
    debugLog(...log: any[]): void;
    enablingPlatfromLogging(): boolean;
}
//# sourceMappingURL=platform.d.ts.map