import { PlatformConfig } from 'homebridge';
/**
 * This is the name of the platform that users will use to register the plugin in the Homebridge config.json
 */
export declare const PLATFORM_NAME = "HoneywellHome";
/**
 * This must match the name of your plugin as defined the package.json
 */
export declare const PLUGIN_NAME = "homebridge-honeywell-home";
/**
 * This is the main url used to access honeywell API
 */
export declare const AuthURL = "https://api.honeywell.com/oauth2/token";
/**
 * This is the main url used to access honeywell API
 */
export declare const LocationURL = "https://api.honeywell.com/v2/locations";
/**
 * This is the main url used to access honeywell API
 */
export declare const DeviceURL = "https://api.honeywell.com/v2/devices";
/**
 * This is the url used to access UI Login to honeywell API
 */
export declare const UIurl = "https://homebridge-honeywell.iot.oz.nu/user/refresh";
export interface HoneywellPlatformConfig extends PlatformConfig {
    credentials?: credentials;
    disablePlugin?: boolean;
    options?: options | Record<string, never>;
}
export declare type credentials = {
    accessToken?: any;
    consumerKey?: any;
    consumerSecret?: any;
    refreshToken?: any;
};
export declare type options = {
    refreshRate?: number;
    pushRate?: number;
    devices?: Array<devicesConfig>;
    logging?: string;
};
export interface devicesConfig extends device {
    deviceClass: string;
    deviceID: string;
    thermostat?: thermostat;
    leaksensor?: leaksensor;
    hide_device?: boolean;
    logging?: string;
    refreshRate?: number;
    retry?: boolean;
}
export declare type thermostat = {
    hide_fan?: boolean;
    hide_humidity?: boolean;
    thermostatSetpointStatus?: string;
    roomsensor?: roomsensor;
    roompriority?: roompriority;
};
export declare type leaksensor = {
    hide_humidity?: boolean;
    hide_temperature?: boolean;
    hide_leak?: boolean;
};
export declare type roomsensor = {
    hide_roomsensor?: boolean;
    hide_temperature?: boolean;
    hide_occupancy?: boolean;
    hide_motion?: boolean;
    hide_humidity?: boolean;
    logging?: string;
    refreshRate?: number;
};
export declare type roompriority = {
    deviceType?: string;
    priorityType?: string;
    logging?: string;
    refreshRate?: number;
};
export declare type modes = {
    Off: number;
    Heat: number;
    Cool: number;
    Auto: number;
};
export declare type payload = {
    mode?: string;
    heatSetpoint: number;
    coolSetpoint: number;
    thermostatSetpointStatus?: string;
    nextPeriodTime?: string;
    autoChangeoverActive?: boolean;
    thermostatSetpoint?: number;
    unit?: string;
};
export declare type location = {
    locationID: number;
    name: string;
    devices: Array<device>;
};
export declare type device = {
    groups?: Array<T9groups>;
    inBuiltSensorState?: inBuiltSensorState;
    settings?: Settings;
    deviceClass: string;
    deviceType: string;
    deviceID: string;
    userDefinedDeviceName: string;
    name?: string;
    isAlive: boolean;
    priorityType?: string;
    units?: string;
    indoorTemperature?: number;
    allowedModes?: string[];
    minHeatSetpoint?: number;
    maxHeatSetpoint?: number;
    minCoolSetpoint?: number;
    maxCoolSetpoint?: number;
    changeableValues?: ChangeableValues;
    operationStatus?: OperationStatus;
    indoorHumidity?: number;
    deviceModel: string;
    displayedOutdoorHumidity?: number;
    scheduleStatus?: string;
    allowedTimeIncrements?: number;
    isUpgrading?: boolean;
    isProvisioned?: boolean;
    macID?: string;
    dataSyncStatus?: string;
    outdoorTemperature?: number;
    deadband?: number;
    hasDualSetpointStatus?: boolean;
    thermostatVersion?: string;
    parentDeviceId?: number;
    service: Service;
    deviceSettings: Record<string, unknown>;
    firmwareVersion?: string;
    vacationHold?: VacationHold;
    currentSchedulePeriod?: CurrentSchedulePeriod;
    scheduleCapabilities?: ScheduleCapabilities;
    scheduleType?: ScheduleType;
    changeSource?: ChangeSource;
    partnerInfo?: PartnerInfo;
    deviceRegistrationDate?: Date;
    indoorHumidityStatus?: string;
    waterPresent: boolean;
    currentSensorReadings: CurrentSensorReadings;
    batteryRemaining: number;
    isRegistered: boolean;
    hasDeviceCheckedIn: boolean;
    isDeviceOffline: boolean;
};
export declare type T9groups = {
    id: number;
};
export declare type inBuiltSensorState = {
    roomId: number;
    roomName: string;
};
export declare type Settings = {
    homeSetPoints?: HomeSetPoints;
    awaySetPoints?: AwaySetPoints;
    fan: Fan;
    temperatureMode?: TemperatureMode;
    specialMode?: SpecialMode;
    hardwareSettings?: HardwareSettings;
    devicePairingEnabled?: boolean;
};
export declare type ChangeableValues = {
    mode: string;
    autoChangeoverActive?: boolean;
    heatSetpoint: number;
    coolSetpoint: number;
    thermostatSetpointStatus?: string;
    nextPeriodTime?: string;
    endHeatSetpoint?: number;
    endCoolSetpoint?: number;
    heatCoolMode: string;
    emergencyHeatActive?: boolean;
};
export declare type OperationStatus = {
    mode: string;
    fanRequest?: boolean;
    circulationFanRequest?: boolean;
};
export declare type Service = {
    mode: string;
};
export declare type AwaySetPoints = {
    awayHeatSP: number;
    awayCoolSP: number;
    smartCoolSP: number;
    smartHeatSP: number;
    useAutoSmart: boolean;
    units: string;
};
export declare type HomeSetPoints = {
    homeHeatSP: number;
    homeCoolSP: number;
    units: string;
};
export declare type TemperatureMode = {
    feelsLike?: boolean;
    air: boolean;
};
export declare type SpecialMode = {
    autoChangeoverActive: boolean;
    emergencyHeatActive?: boolean;
};
export declare type Fan = {
    allowedModes: string[];
    changeableValues: FanChangeableValues;
    fanRunning: boolean;
    allowedSpeeds?: AllowedSpeed[];
};
export declare type FanChangeableValues = {
    mode: string;
};
export declare type AllowedSpeed = {
    item: string;
    value: Value;
};
export declare type Value = {
    speed?: number;
    mode: string;
};
export declare type VacationHold = {
    enabled: boolean;
};
export declare type CurrentSchedulePeriod = {
    day: string;
    period: string;
};
export declare type ScheduleCapabilities = {
    availableScheduleTypes: string[];
    schedulableFan: boolean;
};
export declare type ScheduleType = {
    scheduleType: string;
    scheduleSubType: string;
};
export declare type ChangeSource = {
    by: string;
    name: string;
};
export declare type HardwareSettings = {
    brightness: number;
    maxBrightness: number;
};
export declare type PartnerInfo = {
    singleOrMultiODUConfiguration: number;
    parentDeviceModelId: number;
    parentDeviceBrandId: number;
    oduName: string;
};
export declare type DeviceSettings = {
    temp: Temp;
    humidity: Humidity;
    userDefinedName: string;
    buzzerMuted: boolean;
    checkinPeriod: number;
    currentSensorReadPeriod: number;
};
export declare type Humidity = {
    high: Record<string, unknown>;
    low: Record<string, unknown>;
};
export declare type CurrentSensorReadings = {
    temperature: number;
    humidity: number;
};
export declare type Temp = {
    high: Record<string, unknown>;
    low: Record<string, unknown>;
};
export declare type sensorAccessory = {
    accessoryId: number;
    accessoryAttribute: accessoryAttribute;
    accessoryValue: accessoryValue;
    roomId: number;
    deviceID: string;
};
export declare type accessoryAttribute = {
    type: string;
    connectionMethod: string;
    name: string;
    model: string;
    serialNumber: string;
    softwareRevision: string;
    hardwareRevision: string;
};
export declare type accessoryValue = {
    coolSetpoint: number;
    heatSetpoint: number;
    indoorHumidity: number;
    indoorTemperature: number;
    motionDet: boolean;
    occupancyDet: boolean;
    excludeTemp: boolean;
    excludeMotion: boolean;
    pressure: number;
    occupancyTimeout: number;
    status: string;
    batteryStatus: string;
    rssiAverage: number;
    occupancySensitivity: string;
};
export declare type Priority = {
    deviceId: string;
    status: string;
    currentPriority: CurrentPriority;
};
export declare type CurrentPriority = {
    priorityType: string;
    selectedRooms: Record<string, unknown>;
    rooms: PriorityRooms[];
};
export declare type PriorityRooms = {
    rooms: PriorityRoom;
};
export declare type PriorityRoom = {
    id: number;
    roomName: string;
    roomAvgTemp: number;
    roomAvgHumidity: number;
    overallMotion: boolean;
    accessories: Accessory[];
};
export declare type Accessory = {
    id: number;
    type: string;
    excludeTemp: boolean;
    excludeMotion: boolean;
    temperature: number;
    status: string;
    detectMotion: boolean;
};
export interface AxiosRequestConfig {
    params?: Record<string, unknown>;
    headers?: any;
}
//# sourceMappingURL=settings.d.ts.map